/* 
See README.
This file contains the following routines:

main  - upper driver routine, which runs evol_mdot_in()
evol_mdot_in  - reads/sets parameters and runs other routines

  ---------------------------------------------------------------
  |          MAIN PARAMETERS are set in the file with parameters
  |
  |
  |     mod->mass - mass of the central object [g]
  |     mod->mdisk - mass of the disk at t=0 [g]
  |     mod->r_out_disk - outer radius of the disk [cm]
  |     mod->l  - the order of the Bessel functions
  |     mod->r_ign_in - inner radius of the disk at t=0 [r_out_disk]
  |     mod->r_ign_out - outer radius of the disk at t=0 [r_out_disk]
  |     mod->sigma_index - index in power dependence Sigma from radius
  |      There are two alternative ways to set the viscosity 
  |      parameter normalization nu_0:
  |     1) mod->texp - e-folding time at the decay stage 
  |     2) mod->kappa  - parameter introduced by LP74
  |
  |     MIN_TIME - minimum time to calculate fisk evolution [s] 
  |     MAX_TIME - maximum time to calculate fisk evolution [s] 
  |     NUM_TIME - number of log steps in time
  |    
  |       flags:
  |     mod.flag_outer - flag to treat accretion aret at Rout
  |              it is set in main()
  |         0 is default  -  zero outer accretion rate
  |         1 - constant accretion rate at the outer radius
  |         2 - variable accretion rate at the outer radius
  |
  |     save=1  distribution F(x) is saved for each time, which is
  |               very time consuming
  |     NAME of the model 
   ------------------------------------------------------------------
  |       Additional parameters:
  |     mod->len - number of terms in the series 
  |           If number of terms in the sum is not enough
  |           for small t, then solution becomes stochastic, 
  |           i.e. with negative accretion rate
  |
  |     PRECISION - precision for numerical algorithms
  |     MINX - minimum x
  |     
  |     
   ------------------------------------------------------------------
  
  
  

mdot_in_t1, mdot_in_t - calculate inner accretion rate

save_F_t  - saves F(h) for current t
F_xi_t - returns distribution F(xi) for current t
F_x_t - returns distribution F(x) for current t

integrant - is the expression under
the integral sign to calculate F(x,t) =  int[ F0(x1) * G(x,x1,t) d{x1}]

sigma_init_r - calculates initial Sigma(r) from the disk mass
calc_F_init - initial F0(x1) calculated from initial Sigma(r)
set_initial_F - fills array F0(x1) with values from calc_F_init
F_initial - gets one value of initial F0(x1)

Green_x_x1_t - calculates G(x,x1,t) or  G_dotM(x,x1,t) - Green 
                  functions for F or dotM
numeric_mdot_Green_x_x1_t - calculates numerically derivative of G(x,x1,t)
                        to get G_dotM(x,x1,t)
mod_Green_xi_x1_t - household G(xi,x1,t) for use by numeric_mdot_Green_x_x1_t


integral_green_over_t -  returns the value of the doouble integral 
                  in Eq. (34)  for the constant dotM at rout
get_value_integral_green_over_tau - calculates specific values of 
               integral_green_over_t by interpolation
integrant_green_over_t - to calculate integral_green_over_t


sum_time_int  - the sum of the integrals over time 
         - to calculate double integral
         in Eq. (34) for the case of the variable dotM at rout
calc_time_int_i - returns one integral over time, which is a term 
         for sum_time_int
integrant_tii - to calculate calc_time_int_i


mdot_out_m94   - variable transfer rate at rout 
d_mdot_out_d_t_m94  - derivative of mdot_out_m94

write_model - saves parameters in the output file's header

*/

#define _MAIN_
#include "head_linevdibs.v3.h"
#define ERRORPOINT 100

// precision for numerical algorithms
double PRECISION = 1e-3;

// minimum x
double MINX = 0.001;

// r/rout where the inner accretion rate is evaluated
double r_inner_dotM;

// Time interval and number of time steps:    
double MAX_TIME;
double MIN_TIME;
int NUM_TIME;
char filename[MAX_WORD_LENGTH];
char NAME[MAX_WORD_LENGTH]; // model tag

// Set flag whether to save F(x)
int save;
//___________________________________________________________________

int main(int argc, char *argv[]) {

    struct gmodel mod;
      
//    mod.flag_initial=atoi(argv[1]) ;
  
    
    strcpy(filename,argv[1]);
//    sprint(filename);
//    exit(0);  
//    iprint (mod.flag_initial); 


    // Initialize parameters:
    mod.flag_outer=0;  
    /* values for mod.flag_outer:
    0 - no  accretion rate at the outer radius; 
    1 - constant accretion rate at the outer radius
    2 - variable accretion rate at the outer radius
     */
   mod.flag_green=0;   // houshold parameter, initial value 
   for (int j=0; j<NUM_COOR; j++) mod.zero_interp[j]=0; // initial values
   
     
   
   // Calculate evolution of the inner accretion rate: 
   evol_mdot_in(&mod);   
   return 0;
}

//___________________________________________________________________

void evol_mdot_in (struct gmodel * mod) {
// Calculates evolution of the inner accretion rate
   
   // Read the specific parameters for the disk:
   file(IN,filename,"r");
   read_parameters (IN, mod);
   
   // general options:
   mod->init_disk_part=1.;
   mod->r_out_vis= mod->init_disk_part*mod->r_out_disk; //cm
   mod->r_mass_in =  mod->r_ign_in;
   mod->r_mass_out = mod->r_out_disk;
   
   if ((mod->texp==0) && (mod->kappa!=0)) {
      mod->nu0=(16.*sqr(mod->l)
            /(3.*sqr(mod->kappa)*pow(G*mod->mass,1./2./mod->l)));  
      get_zeroes(mod->len, mod->l, mod->zeroes, mod->constl1, mod->constl2);
      mod->texp =  16./3.*sqr(mod->l/mod->zeroes[0])
            *pow(mod->r_out_disk,(1./2./mod->l))/mod->nu0;

   }
   if (mod->flag_outer==2) { // variable outer acccretion rate
      mod->accr_out = &mdot_out_m94;
      mod->d_accr_out_d_t = &d_mdot_out_d_t_m94;; 
   }
   mod->have_integral=0;
   mod->len=50;
   
   // get zeroes of Bessel function: 
   get_zeroes(mod->len, mod->l, mod->zeroes, mod->constl1, mod->constl2);  
   
   
   // calculate viscosity coefficients from t_exp
   mod->nu0 = 16./3.*sqr(mod->l/mod->zeroes[0])
            *pow(mod->r_out_disk,(1./2./mod->l))/mod->texp;
   mod->kappa=sqrt(16.*sqr(mod->l)
            /(3.*mod->nu0*pow(G*mod->mass,1./2./mod->l)));  

   mod->a=1.;    // maximum x
   
   // time normalizations
   MIN_TIME *= mod->texp;
   MAX_TIME *= mod->texp;
   
   // define radius, where the inner accretion rate is to be evaluated
   // as 1) rms 
   // 2) the minimum radius in calculated evolution
   double r_min_calc = mod->r_out_disk*pow(MINX*mod->a, 4.*mod->l);
   double rms = 6.*G*mod->mass/C/C;
   
   if (r_min_calc>=rms) {r_inner_dotM=r_min_calc*1.01/mod->r_out_disk;}
   else {r_inner_dotM=rms/mod->r_out_disk;}
   printf ("Accretion rate is calculated at %.5e cm\n", r_inner_dotM*mod->r_out_disk);
   printf ("Radius of the marginally stable orbit is %.5e cm\n", rms);
   printf ("Press return to start calculation\n"); 
   getchar();

   // Organize file for output    
   
//   sprintf (filename, "mdotin_%d.dat", mod->flag_initial);
   sprintf (filename, "mdotin_%s.dat", NAME);
   sprint(filename);
   printf ("Result will be in %s\n\n", filename);
   
   
   file(OUT,filename,"w+");
   
   // save info about the model:
   write_model (OUT,mod);
   // write columns info:
   fprintf (OUT, "#1=i\n#2=t\n#3=Mdot_in\n#4=err_Mdot_in\n");
   

   // time variable:         
   double t;
   // number of the time step:
//   int ti;
   // size of the logarithmic step over time:
   double step = pow(MAX_TIME/MIN_TIME,1./(NUM_TIME-1.));

   // inner accretion rate
   double mdin, abserr;
   abserr=0.;
   
   
   // One-mode calculation
/*   if ( (mod->flag_initial==8) ||
        (mod->flag_initial==10) ||
         (mod->flag_initial==11) )   {
 */     
      // get initial distribution of F(h):
      set_initial_F (mod);
      for (int i=0; i<NUM_TIME; i++) {
         
         t=MIN_TIME*pow(step,(double)i);
         printf(" Now t=%.4f s =%.4f days = %.4f * texp\n", 
               t, t/DEN,t/mod->texp);
         
         if (save>0)  save_F_t (t,mod,i);

         // mdot_in_t1 is better for the regime of the variable outer mass supply
         // than mdot_in_t
         double mdin=mdot_in_t1 (t,&abserr,mod);
         
         dprint(mdin);
         printf("_________________________________\n");
         fprintf (OUT, "%d\t%e\t%e\t%e\n", i, t, mdin,abserr);
      }
  // }
   
   fclose(OUT);
   printf ("Finished. Result is in %s\n\n", filename);
   
}

//___________________________________________________________________

void save_F_t( double t, struct gmodel * mod, int ti) {
   printf ("\nsave_F_t begins\n");
   double x[NUM_COOR];
   double F[NUM_COOR];
   double rad[NUM_COOR];
   make_mesh (MINX*mod->a, mod->a, NUM_COOR, 0, x);
   
   char name[MAX_WORD_LENGTH];
   sprintf(name, "F_x%d.dat",ti);
   file (OUT, name,"w+");
   write_model (OUT,mod);
   fprintf (OUT, "#1=j\n#2=x[j]\n#3=r radius\n#4=h angular momentum\n#5=F torque\n#6=F0\n#7=t\n");
   printf ("Save F(x,t) distribution to %s\n", name);
   
   double b = 2.-1./2./mod->l;
   for (int j=0; j<NUM_COOR; j++) {
      double r = mod->r_out_vis*pow(x[j],4.*mod->l);
      rad[j] = r;
      double h = sqrt(G*mod->mass*mod->r_out_vis)*pow(x[j],2.*mod->l);
      double nut = mod->nu0*pow(r,b);
      F[j] = F_x_t (x[j], t, mod);
      
      fprintf (OUT, "%d\t%e\t%e\t%e\t%e\t%e\t%e\n",
            j,x[j],r,h,F[j],mod->zero_interp[j],t);
   }
   fclose(OUT);
}

//___________________________________________________________________

double integrant (double x1, void * p) {
    double f;
    struct my_Green_params * params = (struct my_Green_params *)p;
//  NOTE:  struct my_Green_params { double t;double x1;struct gmodel  * mod ; };
    double t = (params->t); // parameter
    double x = (params->x1);
    struct gmodel * mod = (params->mod);
    double initial_F = F_initial(x1, mod);
    
    // get Green function  for F(x,t) or mdot depending on mod->flag_green
    double green = Green_x_x1_t(x, t,x1,mod);   
   
    // if zero accretion rate at the outer boundary
    if (mod->flag_outer==0) {f= initial_F * green;}

    // if constant accretion rate at the outer boundary
    if (mod->flag_outer==1) {
       double xi1=pow(x1,2.*mod->l);
       double hout = sqrt(mod->mass*G*mod->r_out_vis);
       f=    (initial_F-0.5*xi1*xi1*(mod->mdot_out/2./PI)*hout) * green 
             + 4.*pow(mod->l/mod->kappa,2.)*(mod->mdot_out/2./PI)
             *pow(hout, 1.-1./mod->l)*pow(x1,4.*mod->l-2.)*
             get_value_integral_green_over_tau(x,x1,t,mod,1.) ;
       // if mdot_out=0, the last formula automatically equals to the case flag_outer=0
       };    
    
    // variable accretion rate at the outer boundary (see Appendix of the paper)
    if (mod->flag_outer==2) {
       double xi1=pow(x1,2.*mod->l);
       double hout = sqrt(mod->mass*G*mod->r_out_vis);
       double dotMout0 = (*mod->accr_out)(0.);
       f=    (initial_F-0.5*xi1*xi1*(dotMout0/2./PI)*hout) * green ;
    };     
    return f;   
}

//___________________________________________________________________
   
double get_value_integral_green_over_tau(double x, double x1, double t, 
   struct gmodel * mod, double source_val)  {
   // t is parameter
   int size_xx=50;
   if (mod->have_integral==0) {
      
      //calculate  integral(x1) for the current x and t
      printf ("calculate  integral(x1) for the current x and t\n");
      makeda(xx,size_xx);
      make_mesh (MINX*mod->a, mod->a, size_xx, 0, xx);

      for (int j=0; j<size_xx; j++) {
         mod->green_int_over_t_x[j] = 
               integral_green_over_t(x,xx[j],t,mod,source_val);
      }
      mod->have_integral=1;
      deleteda(xx);
   }
   if (mod->have_integral==1) {
      // interpolate calculated integral(x1) to the current x1
      makeda(xx,size_xx); 
      make_mesh (MINX*mod->a, mod->a, size_xx, 0, xx);

       gsl_interp_accel *acc 
         = gsl_interp_accel_alloc ();
       gsl_spline *spline 
         = gsl_spline_alloc (gsl_interp_cspline, size_xx);

       gsl_spline_init (spline, xx, mod->green_int_over_t_x,size_xx);
       
       double interpolated = gsl_spline_eval (spline, x1, acc);

       gsl_spline_free (spline);
       gsl_interp_accel_free (acc);
       deleteda(xx);
       
       return interpolated;
   }
}

//___________________________________________________________________

 double integrant_green_over_t (double tau, void * p) {
    struct my_Green_params1 * params = (struct my_Green_params1 *)p;
    double t = (params->t);
    double x1 = (params->x1);
    double x = (params->x); 
    struct gmodel * mod = (params->mod);
    double source_val = (params->source_val);
    double green = Green_x_x1_t(x, t-tau, x1,mod);
    double result = green *  source_val;
    return result; 
 }
 

//___________________________________________________________________

double texp_m94=30.*DEN;
double mdot_out_m94 (double tau) 
{ // Mineshige 1994 +step (linear enhancement of Mdot during 1 day : jump=2)
   double t1 = 2.*texp_m94-DEN;
   double t2 = 2.*texp_m94;
   double Mdot1 = 1e+18*exp (-t1/texp_m94);
   double Mdot2 = 4e+18*exp (-t2/texp_m94);
   if (tau>=t2) { 
      return  4.*1e+18*exp (-tau/texp_m94);
   } else if (tau>=t1) { 
      return  Mdot1 + (Mdot2-Mdot1)/(t2-t1)*(tau-t1);
   } else  {
      return 1e+18*exp (-tau/texp_m94);
   }
   printf ("Invalid event in mdot_out_m94\n  ");exit (0);
}

//___________________________________________________________________

double d_mdot_out_d_t_m94(double tau) {
   // Mineshige 1994 + step (linear enhancement of Mdot during 1 day)
      double t1 = 2.*texp_m94-DEN;
      double t2 = 2.*texp_m94;
      double Mdot1 = 1e+18*exp (-t1/texp_m94);
      double Mdot2 = 4e+18*exp (-t2/texp_m94);
      if ((tau>=t1) && (tau<t2)) {
         return (Mdot2-Mdot1)/(t2-t1);
         // constant derivative during the rise of Mdot 
      }
}

//___________________________________________________________________

double  integral_green_over_t (double x, double x1, double t, 
   struct gmodel * mod, double source_val) {
   printf ("\ncalculate integral_green_over_t\n");
   
    gsl_integration_workspace * w 
    = gsl_integration_workspace_alloc (1000);
    double igot_result,  error;

    double tau_min=0.; 
    double tau_max=t;

    gsl_function F;
    size_t neval;
    F.function = &integrant_green_over_t;
    struct my_Green_params1 params = {t,x1,x,mod,source_val};
    F.params = &params;

    gsl_integration_qags (&F, tau_min, tau_max, 0, PRECISION, 1000,
                            w, &igot_result, &error); 
    gsl_integration_workspace_free (w);   
    return igot_result;
} 

//___________________________________________________________________

double F_xi_t (double xi, void * p) {
   // returns disribution F(xi)
   // xi =h/hout
   struct my_Green_params2 * params = (struct my_Green_params2 *)p;
   double t = (params->t);
   struct gmodel * mod = (params->mod);
   double x=pow(xi,1./2./mod->l);
   double gxt= F_x_t ( x,  t, mod);
   return  (gxt);
}


//___________________________________________________________________

double F_x_t (double x, double t, struct gmodel  * mod ) {
   printf ("\nCalculate F_x_t at x=%.3f t=%.4f\n", x,t);
   mod->have_integral = 0;

   gsl_integration_workspace * w 
    = gsl_integration_workspace_alloc (1000);
   double Fxt_result, error;
     
   double x1_min=MINX*mod->a; 
   double x1_max=1.;
     
   gsl_function F;
   size_t neval;
   F.function = &integrant;
   mod->flag_green=1; // flag to retrieve green function for F(x,t)
   
   struct my_Green_params params = {t,x,mod};
   F.params = &params;

   int key; key=1;
   gsl_integration_qag (&F, x1_min, x1_max, 0, PRECISION, 1000, key, 
       w, &Fxt_result, &error);
   gsl_integration_workspace_free (w);
   
   if (mod->flag_outer==1) {
      // constant mass inflow at r_out
       double hout = sqrt(mod->mass*G*mod->r_out_vis);
       Fxt_result += pow(x,4.*mod->l)*(mod->mdot_out/2./PI)*hout/2.;
   } else  if (mod->flag_outer==2) {
      // variable mass inflow at r_out
      double hout = sqrt(mod->mass*G*mod->r_out_vis);
      double dotMout = (*mod->accr_out)(t);
      Fxt_result += pow(x,mod->l)*hout/2./PI/sqr(mod->a) 
          * sum_time_int (x,t,mod);
       Fxt_result += pow(x,4.*mod->l)*(dotMout/2./PI)*hout/2.;
   } else if (mod->flag_outer!=0) {
       printf ("Invalid event in F_x_t(). Exit\n"); exit(0);
   }
   return (Fxt_result);
}

//___________________________________________________________________

double sum_time_int (double x, double t, struct gmodel * mod) {
   // returns the sum of the integrals over time 
   printf ("calculate sum_time_int\n");
   
   size_t size;
   size=mod->len;
   
   gsl_mode_t mode;
   double u[size];
   double L1[size];
   double L2[size];
   for (int i=0; i<size; i++) {
      u[i] = mod->zeroes[i]*x;
   }
   gsl_sf_bessel_sequence_Jnu_e (mod->l,mode, size, u);
   
   double sum_time_int_result = 0.;
   for (int i=0; i<size; i++) {
      double int_time_i;
      int_time_i = calc_time_int_i(t,i,mod);
      sum_time_int_result += u[i]*int_time_i;
   }
   return sum_time_int_result;
}

//___________________________________________________________________

double calc_time_int_i(double t, int i, struct gmodel  * mod) {
   // integral over time - a term for sum_time_int
    gsl_integration_workspace * w 
    = gsl_integration_workspace_alloc (1000);
    double tii_result, error;

    double tau_min=0.; 
    double tau_max=t;  
    /* The time argument of the Green
    function in  is  t-tau. At zero argument
    errors are too big due to the finite series used */

    gsl_function F;
    size_t neval;
    F.function = &integrant_tii;
    
    struct my_Green_params4 params = {t, i,  mod};
    F.params = &params;

    int key; key=1;
    gsl_integration_qag (&F, tau_min, tau_max, 0, PRECISION, 1000, key, 
          w, &tii_result, &error);
    
   gsl_integration_workspace_free (w);   

   return tii_result;
}

//___________________________________________________________________

double integrant_tii (double tau, void * p) {
   
    struct my_Green_params4 * params = (struct my_Green_params4 *)p;
    double t = (params->t);
    int i = (params->i);
    struct gmodel * mod = (params->mod);
    
    double hout = sqrt(G*mod->mass*mod->r_out_vis);
    double tvis = sqr(mod->kappa)*pow(hout,1./mod->l);
    
    double ki = mod->zeroes[i];
    double L1 = mod->constl1[i];
    double L2 = mod->constl2[i];
    // see Appendix of the paper
    double result = exp (- (t-tau)*sqr(ki)/tvis) * 
       (L1*8.*sqr(mod->l) * (*mod->accr_out)(tau)/tvis  
       - L2 * (*mod->d_accr_out_d_t)(tau)) ;
    return result; 
} 

//___________________________________________________________________

void calc_F_init (struct gmodel * mod, int nbins, 
      double *F_init, double *x1)
      /*
        F_init is output that is calculated from initial Sigma
        equals with correct dimension to F = g/2/PI,
        g - couple of Lynden-Bell & Pringle
        F - is the value of Shakura & Lyubarski
      */
       {
   for (int j= 0; j <nbins; j++)
      {
        double rj = mod->r_out_vis*pow(x1[j],4.*mod->l);
        double hj = sqrt(G*mod->mass*rj);
        F_init[j] = 8.  * sqr(mod->l/mod->kappa)
              *pow(hj,1.-1./mod->l) *sqr(rj) * sigma_init_r(rj,mod);
      }
}

//___________________________________________________________________

double sigma_init_r( double r, struct gmodel * mod) {
   // calculates Sigma(r) from disk mass and power sigma_index:
   // Sigma \propto r ^ sigma_index
   double sigma;
   double fun;
   /* mod->mdisk is the intial mass of the disk between radii R1 and R2 
   (R1<R2).
   The mass is distributed so that Sigma propto r^index */
   
   // We set specifically here. These parameters can be easily
   // incorporated in the structure gmodel
   double R1 = mod->r_mass_in;
   double R2 = mod->r_mass_out;
   double index =mod->sigma_index;
   
   fun = (index+2.) * pow (r/R2,index+2.)/(1.-pow(R1/R2,index+2.));
  
   sigma = mod->mdisk/2./PI/sqr(r) * fun;
   return (sigma);
}

//___________________________________________________________________

double mod_Green_xi_x1_t (double xi, void * p) {

   struct my_Green_params * params = (struct my_Green_params *)p;
   double t = (params->t);
   double x1 = (params->x1);
   struct gmodel * mod = (params->mod);
   double x=pow(xi,1./2./mod->l);
   return  Green_x_x1_t ( x,  t, x1, mod);
}
//___________________________________________________________________
   
double Green_x_x1_t (double x, double t,double x1,struct gmodel * mod) {
// calculate Green function for F or Mdot for finite disk
    if (mod->flag_green==2)
    {
       // Green function for Mdot
      double abserr;
      mod->flag_green=1;
      double green=numeric_mdot_Green_x_x1_t (x,t,x1,mod,&abserr);    
      mod->flag_green=2;
      return green;
    }
    
    // Green function for F
   size_t size;
   size=mod->len;
   
   // To save time, change number of terms 
   // in the series for the Bessel function
   if (t>mod->texp) {size = 5;}         
  
   gsl_mode_t mode;
   double v[size];
   double u[size];
   double w[size];
   for (int i=0; i<size; i++) {
      v[i] = mod->zeroes[i]*x1;
      if (v[i]==0) {printf ("\n\n Check initial len value or smallest x! Bessel function is not calculated at 0 \n\n"); exit(0);}
      u[i] = mod->zeroes[i]*x;
      w[i] = mod->zeroes[i]*mod->a;
   }
   gsl_sf_bessel_sequence_Jnu_e (mod->l,mode, size, v);
   gsl_sf_bessel_sequence_Jnu_e (mod->l,mode, size, u);
   gsl_sf_bessel_sequence_Jnu_e (mod->l,mode, size, w);
   double Green=0.;
   double green[size];
   
   for (int i=0; i<size; i++) {
      /* s =  k_i^2/kappa^2/h^(1/l);
      ?Multiply denominator and numerator by the same factor to divide
            values around unity:  */
       double s = (sqr(mod->zeroes[i]/mod->kappa))
           * (pow(G*mod->mass*mod->r_out_vis,-1./2./mod->l));
       
        double term_i = exp(-s*t) * v[i] * u[i] / sqr(w[i]);
       Green += term_i;
   }
   double s0 = (sqr(mod->zeroes[0]/mod->kappa))
           * (pow(G*mod->mass*mod->r_out_vis,-1./2./mod->l));

   Green *=   2./sqr(mod->a) * x1 *  pow(x/x1,mod->l);

   return (Green);
}

//___________________________________________________________________

double F_initial (double x, struct gmodel * mod)
{   // get one value of initial F(x1)
   int ONE=1;
   double x1[ONE];
   double g_init[ONE];
   x1[0] = x;
   /*if ((mod->flag_initial==8)  ||   
       (mod->flag_initial==10) ||   
       (mod->flag_initial==11)) {*/
      calc_F_init(mod,ONE,g_init,x1);
      double rj = mod->r_out_vis*pow(x1[0],4.*mod->l);
      if ((rj < mod->r_ign_in) || (rj > mod->r_ign_out)) {
            g_init[0]=0.0;
         }  
   /*}  else 
   {
      printf("the invalid flag_initial=%d in F_initial(). Exit.\n",mod->flag_initial);
      exit(0);
   }*/
   return (g_init[0]);
}

//_____________________________________________________________________

void set_initial_F (struct gmodel * mod)
{  // fills array mod->zero_interp
   
   printf("Begin set_initial_F ()\n");
   double x1[NUM_COOR];
   double g_init[NUM_COOR];
   make_mesh (MINX*mod->a, mod->a, NUM_COOR, 0, x1);
   for (int j=0; j<NUM_COOR; j++) g_init[j]=0;
   
    /*if ((mod->flag_initial==8) || 
       (mod->flag_initial==10) || 
       (mod->flag_initial==11) ) {*/
      calc_F_init(mod,NUM_COOR,g_init,x1);
      for (int j=0; j<NUM_COOR; j++) {
         double rj = mod->r_out_vis*pow(x1[j],4.*mod->l);
         if ((rj < mod->r_ign_in) || (rj > mod->r_ign_out)) {
            g_init[j]=0.0;
         }
      }
      
   /*} else  {
      printf ("Illegal flag_initial=%d in set_initial_F(). Exit\n",mod->flag_initial);
      exit(0);
   }*/
   
   // store important distributions
   printf ("\nsave initial F begins\n");
   char name[MAX_WORD_LENGTH];
//   sprintf(name, "F_x%d.dat",mod->flag_initial);
   sprintf(name, "F_x_%s.dat",NAME);
   file (OUT, name,"w+");
   write_model (OUT,mod);
   fprintf (OUT, "#1=j\n#2=x[j]\n#3=r\n#4=h\n#5=F\n#6=F0 (shifted on x!)\n#7=t\n");
   printf ("Save F(x,t) distribution to %s\n", name);
   
   double b = 2.-1./2./mod->l;
  
   for (int j=0; j<NUM_COOR; j++) {
      double r = mod->r_out_vis*pow(x1[j],4.*mod->l);
      double nut = mod->nu0*pow(r,b);
      double h = sqrt(G*mod->mass*mod->r_out_vis)*pow(x1[j],2.*mod->l);
      
      fprintf (OUT, "%d\t%e\t%e\t%e\t%e\t%e\n",
            j,x1[j],r,h,g_init[j],mod->zero_interp[j]);
      // mod->zero_interp is shifted !
   }
 
   fclose(OUT);
  
   for (int j=0; j<NUM_COOR; j++) { 
     //printf ("g_init[%d]=%.3e ",j,g_init[j]);  
     mod->zero_interp[j] = g_init[j];
   }
}
 
//___________________________________________________________________
 
double mdot_in_t1 (double t, double * abserr_mdot,struct gmodel * mod )
{ // to find mdot_in calculate numerically derivative of F(xi,t) over xi.
   // works identically in regard to result with mdot_in_t
   // time-budget hasn't been checked
   printf ("\nCalculating accretion rate by mdot_in1()\n");
   mod->have_integral = 0; 
   // flag  (1st time for the current t integrant
   // is calculated for the grid of coordinates)
   
   gsl_function F1;
   double mdin1_result;
   double abserr; 
   
   // position where inner accretion rate is evaluated:
   double x =pow(r_inner_dotM,1./4./mod->l);

   double hout = sqrt(G*mod->mass*mod->r_out_vis);
      
    // make the linear fit of several points F(xi) 
   printf ("\n Calculate linear fit for derivative\n");
   struct my_Green_params2 params = {t, mod};
   void  * ppp;
   ppp=&params;
   int fit_size = FITSIZE;
   double F_arr[FITSIZE];
   double xi_arr[FITSIZE] = {0.05,0.07,0.09,0.11,0.13};

    for (int j =0;j<fit_size; j++) {
       F_arr[j] = F_xi_t(xi_arr[j], ppp);
    }
    double c1,  cov11, sumsq;
    gsl_fit_mul (xi_arr, 1, F_arr, 1, fit_size, &c1, &cov11, &sumsq);
    mdin1_result = c1;
   double res_mdot = mdin1_result*2.*PI/hout;   
   return(res_mdot);
}
  
//___________________________________________________________________
   
double mdot_in_t (double t, double * abserr_mdot,struct gmodel * mod ) {
 // works identically in regard to result with mdot_in_t1
   // time-budget hasn't been checked
   printf ("\ncalculating mdot_in\n");
   mod->have_integral = 0;

   /* x_in = (h_in/h_out)^(1/2/l) */
   // coordinate, where the inner accretion rate is evaluated   
   double x =pow(r_inner_dotM,1./4./mod->l);
   
   gsl_integration_workspace * w  = gsl_integration_workspace_alloc (1000);
   double mdin_result, error, result_er, error_er;
   
   double x1_min=MINX*mod->a;
   double x1_max=mod->a;

   gsl_function F;
   size_t neval;
    
   mod->flag_green=2; // get green function for Mdot(x,t)
   F.function = &integrant;
   struct my_Green_params params = {t,x,mod};
   F.params = &params;

    /* If the initial spike is too narrow, then the integrating routine
       can miss it;    the cure is to encrease the key variable;
       Default is key=1. 
       Beware! Program time increases significantly for key>1  */
    int key; key=1;
    
    gsl_integration_qag (&F, x1_min, x1_max, 0, PRECISION, 1000, key, 
          w, &mdin_result, &error);
    
    gsl_integration_workspace_free (w);
    
    double hout = sqrt(G*mod->mass*mod->r_out_vis);
    if (mod->flag_outer==1) {
       // + h *dotMout/2pi:
       mdin_result += pow(x,2.*mod->l)* hout* (mod->mdot_out/2./PI);
    } else if (mod->flag_outer==2) {
       // + h *dotMout/2pi:
       double dotMout = (*mod->accr_out)(t);
       mdin_result += pow(x,2.*mod->l)* hout* (dotMout/2./PI);
    } else if (mod->flag_outer!=0) {
       printf ("Invalid event in mdot_in_t(). Exit\n"); exit(0);
    }
    
    double factor = 2.*PI/hout;
    *abserr_mdot = factor *error;      
   return (factor*mdin_result);
}

//___________________________________________________________________

double numeric_mdot_Green_x_x1_t (double x, double t,double x1,
   struct gmodel * mod, double * abserr ) {
  gsl_function F;
  double result;

  F.function = &mod_Green_xi_x1_t;
  struct my_Green_params params = {t,x1,mod};
  F.params = &params;
  
  if (x>ACCUX*10) {
     gsl_deriv_central (&F, pow(x,2.*mod->l), PRECISION, &result, abserr);
     }
  else {
     gsl_deriv_forward (&F, pow(x,2.*mod->l),  PRECISION, &result, abserr);
     }
  return (result);

}
//___________________________________________________________________

void write_model (FILE *  OUT, struct gmodel * mod) 
{  // routine to save parameters in the file's header
   double b = 2.-1./2./mod->l;
   fprintf (OUT, "#l=%e\n",mod->l);
   fprintf (OUT, "#b=%e\n",b);      
   fprintf (OUT, "#mass=%e\n",mod->mass);
   fprintf (OUT, "#mdisk=%e\n",mod->mdisk);
   fprintf (OUT, "#mdot_out=%e\n",mod->mdot_out);
   fprintf (OUT, "#r_out_disk=%e\n",mod->r_out_disk);
   fprintf (OUT, "#r_out_vis=%e\n",mod->r_out_vis);
   fprintf (OUT, "#r_ign_in=%e\n",mod->r_ign_in);
   fprintf (OUT, "#r_ign_out=%e\n",mod->r_ign_out);
   fprintf (OUT, "#r_mass_out=%e\n",mod->r_mass_out);
   fprintf (OUT, "#r_mass_in=%e\n",mod->r_mass_in);
   fprintf (OUT, "#init_disk_part=%e\n",mod->init_disk_part);
   fprintf (OUT, "#sigma_index=%e\n",mod->sigma_index);
   fprintf (OUT, "#texp(days)=%e\n",mod->texp/DEN);
   fprintf (OUT, "#texp(s)=%e\n",mod->texp);      
   fprintf (OUT, "#nu0=%e\n",mod->nu0); 
   fprintf (OUT, "#nu_t_out=%e\n",mod->nu0*pow(mod->r_out_disk,b));         
   fprintf (OUT, "#len_Bessel=%d\n",mod->len);
   fprintf (OUT, "#a=%e\n",mod->a);
   fprintf (OUT, "#kappa=%e\n",mod->kappa);
   fprintf (OUT, "#texp=%e\n",mod->texp);
//   fprintf (OUT, "#flag_initial=%d\n",mod->flag_initial);      
   fprintf (OUT, "#flag_green=%d\n",mod->flag_green);      
   fprintf (OUT, "#flag_outer=%d\n",mod->flag_outer);     
   fprintf (OUT, "#PRECISION=%e\n", PRECISION);
   fprintf (OUT, "#NUM_COOR=%d\n", NUM_COOR);   
   fprintf (OUT, "#MINX=%e\n", MINX);         
   fprintf (OUT, "#ACCUX=%e\n", ACCUX);                
   fprintf (OUT, "#r_inner_dotM=%e\n", r_inner_dotM);      
};
void read_parameters (FILE *  IN, struct gmodel * mod) {
    
    char field_value[MAX_WORD_LENGTH] ;
    char field_name[MAX_WORD_LENGTH] ;
    
    //initialize field_value with non-empty string:
    strcpy(field_value,"---");
    
    // read parameters from file 
    while (strcmp (field_value,"")!=0) {
       strcpy(field_value,"");
       strcpy(field_name,"");

       fscanf(IN,"%s # %s\n",field_value,field_name);
       if (strcmp (field_name,"")!=0) {
          if (strcmp (field_name,"mass")==0){// object mass [g]
             sscanf(field_value,"%le",&mod->mass);continue;
          }
          if (strcmp (field_name,"mdisk")==0){// disk mass [g]
             sscanf(field_value,"%le",&mod->mdisk);continue;
          }
          if (strcmp (field_name,"r_out_disk")==0){// disk mass [cm]
             sscanf(field_value,"%le",&mod->r_out_disk);continue;
          }
          if (strcmp (field_name,"l")==0){// the order of the Bessel functions
             sscanf(field_value,"%le",&mod->l);continue;
          }      
          if (strcmp (field_name,"r_ign_in")==0){// inner radius of the disk at t=0 [r_out_disk]
             sscanf(field_value,"%le",&mod->r_ign_in);continue;
             
          }
          if (strcmp (field_name,"r_ign_out")==0){// outer radius of the disk at t=0 [r_out_disk]
             sscanf(field_value,"%le",&mod->r_ign_out);continue;
          }
          if (strcmp (field_name,"sigma_index")==0){// index in power dependence Sigma from radius 
             sscanf(field_value,"%le",&mod->sigma_index);continue;
          }
          if (strcmp (field_name,"texp")==0){// e-folding time at the decay stage [days]
             sscanf(field_value,"%le",&mod->texp);continue;
          }
          if (strcmp (field_name,"kappa")==0){// parameter introduced by LP74
             sscanf(field_value,"%le",&mod->kappa);continue;
          }
          if (strcmp (field_name,"MIN_TIME")==0){// minimum time to calculate disk evolution [texp] 
             sscanf(field_value,"%le",&MIN_TIME);continue;
          }
          if (strcmp (field_name,"MAX_TIME")==0){// maximum time to calculate disk evolution [texp] 
             sscanf(field_value,"%le",&MAX_TIME);continue;
          }
           if (strcmp (field_name,"NUM_TIME")==0){ // number of log steps in time
            sscanf(field_value,"%d",&NUM_TIME);continue;
         }
         if (strcmp (field_name,"save")==0){ // 1 ->distribution F(x) is saved for each time, which is
//          very time consuming
            sscanf(field_value,"%d",&save);continue;
         }
         if (strcmp (field_name,"flag_outer")==0){ // flag to treat accretion aret at Rout
            sscanf(field_value,"%d",&mod->flag_outer);continue;
         }
         if (strcmp (field_name,"name")==0){ // flag to treat accretion aret at Rout
            sscanf(field_value,"%s",NAME);continue;
         }
       } // if
    } //while  
   
    // normalizations:
    mod->r_ign_in *= mod->r_out_disk;
    mod->r_ign_out *=mod->r_out_disk;
    mod->texp *=DEN;
    
   fclose(IN);
}

#undef _MAIN_
#undef ERRORPOINT



